package org.apache.geronimo.samples.brokerage.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import org.apache.geronimo.samples.brokerage.bo.Stock;
import org.apache.geronimo.samples.brokerage.bo.User;
import org.apache.geronimo.samples.brokerage.exception.DBException;


public class TradeDAO {

	DataSource ds;
	
	public TradeDAO() throws Exception {
		try {
			InitialContext initialContext = new InitialContext();
			ds = (DataSource) initialContext.lookup("java:comp/env/jdbc/TradeDB");
		} catch (Exception ex) {
			throw new Exception("Couldn't open connection to database: "+ ex.getMessage());
		}
	}

	public User getUserByUserId(String userId) throws DBException {

		User user = null;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "select * from Users where userid = ?";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setString(1, userId);
			
			ResultSet rs = pStmt.executeQuery();
			if (rs.next()) {
				user = new User(rs.getString("userid"), rs.getString("name"), rs.getString("password"), rs.getString("address"), rs.getFloat("cash"));
			}
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		return user;
	}

	public List getUserStocks(String userId) throws DBException {
		List stocks = new ArrayList();
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "select A.quantity,A.stockid,B.name,B.price from TradingAccount A,Stocks B where A.userid = ? and A.stockid = B.id";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setString(1, userId);
			Stock stock = null;
			ResultSet rs = pStmt.executeQuery();
			while (rs.next()) {
				stock = new Stock(rs.getString(2), rs.getString(3), rs.getFloat(4), rs.getInt(1));
				stocks.add(stock);
			}
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		return stocks;
	}

	public List getStocks() throws DBException {
		List stocks = new ArrayList();
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "select * from Stocks";
			PreparedStatement pStmt = con.prepareStatement(query);

			Stock stock = null;
			ResultSet rs = pStmt.executeQuery();
			while (rs.next()) {
				stock = new Stock(rs.getString(1), rs.getString(2), rs.getFloat(3), 0);
				stocks.add(stock);
			}
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		return stocks;
	}

	public float getUserCash(String userId) throws DBException {
		float cash = 0;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "select cash from Users where Userid = ?";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setString(1, userId);
			ResultSet rs = pStmt.executeQuery();
			while (rs.next()) {
				cash = rs.getFloat("cash");
			}
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		return cash;
	}

	public boolean buyStock(String userId, String stockId, int quantity) throws DBException {
		float cashAvailable = getUserCash(userId);
		float costOfStock = getStockPrice(stockId);
		float totalCost = costOfStock * quantity;
		int availableStock = getStockQuantityForUser(userId, stockId);
		cashAvailable = cashAvailable - totalCost;
		availableStock = availableStock + quantity;
		Connection con = null;
		if (cashAvailable >= 0 && (availableStock != quantity)) {
			try {
				con = ds.getConnection();
				con.setAutoCommit(false);
				if (setUserCash(userId, cashAvailable) && setUserStock(userId, stockId, availableStock)) {
					con.commit();
				} else {
					con.rollback();
				}
				con.setAutoCommit(true);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new DBException(e.getMessage());
			}finally {
				if(con != null){
					try {
						con.close();
					} catch (SQLException e) {
						e.printStackTrace();
					}
				}
			}
		} else if (cashAvailable >= 0 && (availableStock == quantity)) {
			try {
				con = ds.getConnection();
				con.setAutoCommit(false);
				if (setUserCash(userId, cashAvailable) && addUserStock(userId, stockId, availableStock)) {
					con.commit();
				} else {
					con.rollback();
				}
				con.setAutoCommit(true);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new DBException(e.getMessage());
			}finally {
				if(con != null){
					try {
						con.close();
					} catch (SQLException e) {
						e.printStackTrace();
					}
				}
			}
		} else {
			return false;
		}
		return true;
	}

	public boolean sellStock(String userId, String stockId, int quantity) throws DBException {
		float cashAvailable = getUserCash(userId);
		float costOfStock = getStockPrice(stockId);
		float totalCost = costOfStock * quantity;
		int availableStock = getStockQuantityForUser(userId, stockId);
		cashAvailable = cashAvailable + totalCost;
		availableStock = availableStock - quantity;
		Connection con = null;
		if (availableStock >= 0) {
			try {
				con = ds.getConnection();
				con.setAutoCommit(false);
				if (setUserCash(userId, cashAvailable) && setUserStock(userId, stockId, availableStock)) {
					con.commit();
				} else {
					con.rollback();
				}
				con.setAutoCommit(true);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new DBException(e.getMessage());
			}finally {
				if(con != null){
					try {
						con.close();
					} catch (SQLException e) {
						e.printStackTrace();
					}
				}
			}

		} else {
			return false;
		}
		return true;
	}

	public float getStockPrice(String stockId) throws DBException {
		float stockPrice = 0f;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "select price from Stocks where id = ?";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setString(1, stockId);
			ResultSet rs = pStmt.executeQuery();
			while (rs.next()) {
				stockPrice = rs.getFloat("price");
			}
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}

		return stockPrice;
	}

	private boolean setUserCash(String userId, float cash) throws DBException {
		int status = 0;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "update Users set cash = ? where userid = ?";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setFloat(1, cash);
			pStmt.setString(2, userId);
			status = pStmt.executeUpdate();
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		if (status == 0) {
			return false;
		} else {
			return true;
		}
	}

	// buy = true sell = false
	private boolean setUserStock(String userId, String stockId, int quantity) throws DBException {
		int existingQuantity = 0;
		int status = 0;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "update TradingAccount set quantity = ? where stockid = ? and userid = ?";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setInt(1, quantity);
			pStmt.setString(2, stockId);
			pStmt.setString(3, userId);
			status = pStmt.executeUpdate();
			pStmt.close();
			query = "delete from TradingAccount where quantity = ?";
			pStmt = con.prepareStatement(query);
			pStmt.setInt(1, 0);
			pStmt.executeUpdate();
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		if (status == 0) {
			return false;
		} else {
			return true;
		}

	}

	private boolean addUserStock(String userId, String stockId, int quantity) throws DBException {

		int status = 0;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "insert into TradingAccount values (?,?,?)";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setInt(3, quantity);
			pStmt.setString(2, stockId);
			pStmt.setString(1, userId);
			status = pStmt.executeUpdate();
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		if (status == 0) {
			return false;
		} else {
			return true;
		}

	}

	public int getStockQuantityForUser(String userId, String stockId) throws DBException {
		int existingQuantity = 0;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "select quantity from TradingAccount where userid = ? and stockid = ?";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setString(1, userId);
			pStmt.setString(2, stockId);
			ResultSet rs = pStmt.executeQuery();
			while (rs.next()) {
				existingQuantity = rs.getInt("quantity");
			}
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		return existingQuantity;
	}

	public boolean addUser(String userId, String name, String password, String address, float cash) throws DBException {
		int status = 0;
		Connection con = null;
		try {
			con = ds.getConnection();
			String query = "insert into Users values (?,?,?,?,?)";
			PreparedStatement pStmt = con.prepareStatement(query);
			pStmt.setString(1, userId);
			pStmt.setString(2, name);
			pStmt.setString(3, password);
			pStmt.setString(4, address);
			pStmt.setFloat(5, cash);
			status = pStmt.executeUpdate();
			pStmt.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
			throw new DBException(ex.getMessage());
		}finally {
			if(con != null){
				try {
					con.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		if (status == 0) {
			return false;
		} else {
			return true;
		}
	}
	
}
